/****************************************************************************

    player.c
    player handling

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/


#include "doubledash.h"
#include "player.h"
#include <stdio.h>
#include <stdlib.h>


#define PLAYER_VIEW_STEP 0.38

typedef struct {
  int level_score;
  int total_score;
  int x, y;
  double view_x;
  double view_y;
  int move_dir;
  int digging;
  int up;
  int down;
  int right;
  int left;
  int dig;
  int can_move;
  int flash;
  int success;
  int dead;
} Player;

Player players[4];


/*
  set player keys
 */
void player_set_keys( int id, int up, int right, int down, int left, int dig )
{
  players[id].up=up;
  players[id].left=left;
  players[id].right=right;
  players[id].down=down;
  players[id].dig=dig;
}


/*
  check keypress
 */
void player_keypress( int key )
{
  int id;

  for( id=0; id<4; ++id )
    {
      if( players[id].up==key )
	players[id].move_dir=1;
      
      if( players[id].right==key )
	players[id].move_dir=2;
      
      if( players[id].down==key )
	players[id].move_dir=3;
      
      if( players[id].left==key )
	players[id].move_dir=4;
      
      if( players[id].dig==key )
	players[id].digging=1;
    }
}

/*
  check keydepress
 */
void player_keydepress( int key )
{

  int id;

  for( id=0; id<4; ++id )
    {
      if( players[id].up==key && players[id].move_dir==1 )
	players[id].move_dir=0;
      
      if( players[id].right==key && players[id].move_dir==2 )
	players[id].move_dir=0;
      
      if( players[id].down==key && players[id].move_dir==3 )
	players[id].move_dir=0;

      if( players[id].left==key && players[id].move_dir==4 )
	players[id].move_dir=0;

      if( players[id].dig==key )
	players[id].digging=0;
    }
}


/*
  init player in a new level
 */
void player_init( int id, int x, int y, int view_x, int view_y )
{
  players[id].level_score=0;
  players[id].x=x;
  players[id].y=y;
  players[id].view_x=view_x;
  players[id].view_y=view_y;
  players[id].move_dir=0;
  players[id].digging=0;
  players[id].flash=0;
  players[id].success=0;
  players[id].dead=0;
}


/*
  null players score
 */
void player_null_score()
{
  players[0].total_score=0;
  players[1].total_score=0;
  players[2].total_score=0;
  players[3].total_score=0;
}




/*
  get players view center
 */
void player_get_view( int id, int *viewx, int *viewy, int titlew, int titleh )
{
  *viewx=players[id].view_x*titlew;
  *viewy=players[id].view_y*titleh;
}


/*
  Do player next frame
 */
void player_next_frame( int id, int major )
{
  if( players[id].view_x<players[id].x )
    {
      players[id].view_x+=PLAYER_VIEW_STEP;
      if( players[id].view_x>players[id].x ) 
	players[id].view_x=players[id].x;
    }
  if( players[id].view_x>players[id].x )
    {
      players[id].view_x-=PLAYER_VIEW_STEP;
      if( players[id].view_x<players[id].x ) 
	players[id].view_x=players[id].x;
    }
  if( players[id].view_y<players[id].y )
    {
      players[id].view_y+=PLAYER_VIEW_STEP;
      if( players[id].view_y>players[id].y ) 
	players[id].view_y=players[id].y;
    }
  if( players[id].view_y>players[id].y )
    {
      players[id].view_y-=PLAYER_VIEW_STEP;
      if( players[id].view_y<players[id].y ) 
	players[id].view_y=players[id].y;
    }

  if( !major ) return;	

  if( players[id].flash>0 )
    players[id].flash++;
  if( players[id].flash==5 )
    players[id].flash=0;

}



/*
  Get player score
 */
int player_get_score( int id )
{
  return players[id].level_score;
}


/*
  Get player total score
 */
int player_get_total_score( int id )
{
  return players[id].total_score;
}

/*
  Add player total score
 */
void player_add_total_score( int id, int value )
{ 
  players[id].total_score+=value;
}





/*
  Get player movement direction
*/
void player_get_dir( int id, int *dx, int *dy, int *dig )
{
  *dig=players[id].digging;
  switch( players[id].move_dir )
    {
    case 0:
      *dx=0;
      *dy=0;
      break;

    case 1:
      *dx=0;
      *dy=-1;
      break;

    case 2:
      *dx=1;
      *dy=0;
      break;

    case 3:
      *dx=0;
      *dy=1;
      break;

    case 4:
      *dx=-1;
      *dy=0;
      break;

    default:
      break;
    }
}


/*
  Let player move
*/
void player_move( int id )
{
  int dx, dy, dig;
  player_get_dir( id, &dx, &dy, &dig );
  players[id].x+=dx;
  players[id].y+=dy;
}


/*
  Player got diamond
*/
void player_got_diamond( int id )
{
  players[id].level_score++;
}


/*
  Player can move flag
*/
void player_can_move( int id, int value )
{
  players[id].can_move=value;
}


/*
  Get player id
*/
int player_get_id( int x, int y )
{
  int id;
  for( id=0; id<4; ++id )
    if( x==players[id].x && y==players[id].y && !players[id].dead )
      return id;

  fprintf( stderr, "DoubleDash: Fatal error. Player lost." );
  exit(1);

  return 0;
}



/*
  Has player succeded?
*/
int player_get_success( int id )
{
  return players[id].success;
}


/*
  Player succeded
*/
void player_set_success( int id )
{
  players[id].flash=1;
  players[id].success=1;
}


/*
  Player get flash state
*/
int player_get_flash( int id )
{
  return players[id].flash;
}


/*
  Get player frame
*/
int player_frame( int id, int frame )
{
  int dir=players[id].move_dir;
  int move=players[id].can_move;
  
  if( move )
    {
      if( dir==1 || dir==2 )
	return 3*8+(frame/2)%4;
      if( dir==3 || dir==4 )
	return 3*8+4+(frame/2)%4;
    }

  if( dir==2 ) return 8+(frame/2)%8;
  if( dir==4 ) return 2*8+(frame/2)%8;

  return (frame/2)%8;
}




/*
  Player is dead
 */
void player_dead( int id )
{
  players[id].dead=1;
}



